{% load i18n sizeformat %}

<div class="detail">
  <dl class="dl-horizontal">
    <dt>{% trans "Name" %}</dt>
    <dd>{{ fleet.name }}</dd>
    <dt>{% trans "ID" %}</dt>
    <dd>{{ fleet.uuid }}</dd>
    <dt>{% trans "Description" %}</dt>
    <dd>{{ fleet.description }}</dd>
  </dl>

  <h4>{% trans "Boards" %}</h4>
    <hr class="header_rule">
    <dl class="dl-horizontal">
      {% if boards %}
          {% for board in boards %}
              <dt>{{ board.name }}</dt>
              <dd>{{ board.uuid }}</dd>
          {% endfor %}
      {% else %}
          <dd>--</dd>
      {% endif %}
    </dl>
</div>
