{% load i18n sizeformat %}

<div class="detail">
  <dl class="dl-horizontal">
    <dt>{% trans "Name" %}</dt>
    <dd>{{ webservice.name }}</dd>
    <dt>{% trans "Port" %}</dt>
    <dd>{{ webservice.port }}</dd>
    <dt>{% trans "ID" %}</dt>
    <dd>{{ webservice.uuid }}</dd>
    <dt>{% trans "Created At" %}</dt>
    <dd>{{ webservice.created_at }}</dd>
    <dt>{% trans "extra" %}</dt>
    <dd>{{ webservice.extra }}</dd>
  </dl>

  <h4>{% trans "Board" %}</h4>
    <hr class="header_rule">
    <dl class="dl-horizontal">
      {% if board_uuid %}
          <dt>{{ board_name }}</dt>
          <dd>{{ board_uuid }}</dd>
      {% else %}
          <dd>--</dd>
      {% endif %}
    </dl>
</div>
