{% load i18n sizeformat %}

<div class="detail">
  <dl class="dl-horizontal">
    <dt>{% trans "Name" %}</dt>
    <dd>{{ service.name }}</dd>
    <dt>{% trans "ID" %}</dt>
    <dd>{{ service.uuid }}</dd>
    <dt>{% trans "Protocol" %}</dt>
    <dd>{{ service.protocol }}</dd>
    <dt>{% trans "Port" %}</dt>
    <dd>{{ service.port }}</dd>
    <dt>{% trans "Extra" %}</dt>
    <dd>{{ service.extra }}</dd>
  </dl>
</div>
