{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Remove plugin(s) from this board." %}{% endblock %}

{% block main %}
    {% include 'iot/boards/_removeplugins.html' %}
{% endblock %}
