{% load i18n sizeformat %}

<div class="detail">
  <dl class="dl-horizontal">
    <dt>{% trans "Name" %}</dt>
    <dd>{{ plugin.name }}</dd>
    <dt>{% trans "ID" %}</dt>
    <dd>{{ plugin.uuid }}</dd>
    <dt>{% trans "Public" %}</dt>
    <dd>{{ plugin.public }}</dd>
    <dt>{% trans "Callable" %}</dt>
    <dd>{{ plugin.callable }}</dd>
    <dt>{% trans "Code" %}</dt>
    <dd><pre>{{ plugin.code }}</pre></dd>
  </dl>
</div>
